#!/bin/sh
###############################################
#        Patch script for SDSU v2.1.0
#                    Hiro Masunaga, Sep 7, 2009
###############################################

echo "This script will upgrade the SDSU v2.0 to v2.1.0."
echo "If you don't have the SDSU version 2 installed at all,"
echo "please cancel this script and download the SDSU-v2.1.0 package."
echo "Do you have an earlier version of SDSU-v2? (y/n)"
read yesno
if [ "$yesno" = "n" ]; then
     exit
fi
PTCHPATH=`pwd`
SDSUPATH=`cd ..; pwd`
echo "It is assumed that the path to your SDSU is "
echo $SDSUPATH
echo "Is this correct?(y/n) If no, provide your SDSU path."
read yesno
if [ "$yesno" = "n" ]; then
     echo "Please type the FULL path to your SDSU."
     read SDSUPATH
fi
#
echo "--------- Main Simulator Units -----------"
cd $PTCHPATH/src/simulator
MAIN=$SDSUPATH/src/simulator
GASD=gas
HYDR=hydromet
MCRW=microw
RADR=radar
VSIR=vis_IR
#
echo "      ### User Interface Routines ###"
echo "For each routine, you will be asked whether the existing file"
echo "may be overwritten. If you answer no, a separate file"
echo "(*.f90_v2.1.0) will be created to store the copy."
#
for file in param_set.f90 $HYDR/microp_set.f90
do
echo "Overwrite "$file"? (y/n)"
read yesno
if [ "$yesno" = "y" ]; then
     \cp $file $MAIN/$file
else
     \cp $file $MAIN/${file}_v2.1.0
fi
done
#
echo "      ### Makefile, rd_CRM, and F90 Modules ###"
echo "For each file, you will be asked if you want a backup file created."
echo "If you answer yes, the user's module files will be duplicated as"
echo "*.f90_bkup before they are copied."
#
for file in makefile rd_CRM.f90 MAINparameters.f90 */*parameters.f90
do
echo "Back up "$file"? (y/n)"
read yesno
if [ "$yesno" = "y" ]; then
     \mv $MAIN/$file $MAIN/${file}_bkup
fi
\cp $file $MAIN/$file
done
#
echo "       ### Internal Routines ###"
echo "Other internal routines are usually not edited by users and"
echo "will be replaced without being backed up. If you don't want" 
echo "any of them overwritten, exit now and back them up before you"
echo "proceed further. See README for the list of the files to be replaced."
echo "Ready to proceed?(y/n)"
read yesno
if [ "$yesno" = "n" ]; then
     exit
fi
#
cd $MAIN
if [ ! -e beamconv ]; then
     ln -s ../beamconv beamconv
fi
cd $PTCHPATH/src/simulator
#
for file in dealloc_CRM.f90 
do
echo "copying "$file
\cp $file $MAIN/$file
done
for file in $GASD/absorb.f
do
echo "copying "$file
\cp $file $MAIN/$file
done
for file in $HYDR/heymsplatt.f90 $HYDR/mie_*.f90
do
echo "copying "$file
\cp $file $MAIN/$file
done
for file in $MCRW/SIMmicro.f90 $MCRW/*lut_mwr.f90 $MCRW/README-microw
do
echo "copying "$file
\cp $file $MAIN/$file
done
for file in $RADR/SIMradar.f90 $RADR/*lut_rdr.f90 $RADR/README-radar $RADR/radar_echo.f90
do
echo "copying "$file
\cp $file $MAIN/$file
done
for file in $VSIR/SIMvisir.f90 $VSIR/*lut_vir.f90 $VSIR/lib_*.f90 $VSIR/rstr6b.f90
do
echo "copying "$file
\cp $file $MAIN/$file
done
echo "copying plug-ins/"
\cp -r plug-ins $MAIN/
#
echo "--------- Mie LUT Generator -----------"
cd $PTCHPATH/src/MieLUT
MLUT=$SDSUPATH/src/MieLUT
#
echo "      ### User Interface Routines ###"
echo "For each routine, you will be asked whether the existing file"
echo "may be overwritten. If you answer no, a separate file"
echo "(*.f90_v2.1.0) will be created to store the copy."
#
for file in paramLUT_set.f90
do
echo "Overwrite "$file"? (y/n)"
read yesno
if [ "$yesno" = "y" ]; then
     \cp $file $MLUT/$file
else
     \cp $file $MLUT/${file}_v2.1.0
fi
done
#
echo "      ###  F90 Modules ###"
echo "For each file, you will be asked if you want a backup file created."
echo "If you answer yes, the user's module files will be duplicated as"
echo "*.f90_bkup before they are copied."
#
for file in LUTparameters.f90
do
echo "Back up "$file"? (y/n)"
read yesno
if [ "$yesno" = "y" ]; then
     \mv $MLUT/$file $MLUT/${file}_bkup
fi
\cp $file $MLUT/$file
done
#
echo "       ### Internal Routines ###"
echo "Other internal routines are usually not edited by users and"
echo "will be replaced without being backed up. If you don't want" 
echo "any of them overwritten, exit now and back them up before you"
echo "proceed further. See README for the list of the files to be replaced."
echo "Ready to proceed?(y/n)"
read yesno
if [ "$yesno" = "n" ]; then
     exit
fi
#
for file in mktab_*f90
do
echo "copying "$file
\cp $file $MLUT/$file
done
#
echo "--------- Mie LUT -----------"
cd $SDSUPATH/Mie-table
if [ ! -e exp2mo ]; then
     mkdir exp2mo
fi
mkdir $SDSUPATH/Mie-table/lgn2mo
if [ ! -e lgn2mo ]; then
     mkdir lgn2mo
fi
#
cd $PTCHPATH/Mie-table/melt-p
MLUT=$SDSUPATH/Mie-table/melt-p
echo "       ### Mie LUT for melting particles ###"
echo "The Mie lookup tables for melting particles have been updated."
echo "The default LUTs in Mie-table/melt-p will be replaced. "
echo "If you have created your own LUT, you may need to run the"
echo "Mie LUT generator again before the main simulator is used."
echo "Ready to proceed?(y/n)"
read yesno
if [ "$yesno" = "n" ]; then
     exit
fi
\cp * $MLUT/.
#
